@@ -7,13 +7,14 @@
 #
 # @author Davide Brunato <brunato@sissa.it>
 #
-from typing import cast, Any, Optional
+from typing import Any, Optional
 
 from ..helpers import QNAME_PATTERN
-from .atomic_types import AtomicTypeMeta
+from .atomic_types import AnyAtomicType
+from .untyped import UntypedAtomic
 
 
-class AbstractQName(metaclass=AtomicTypeMeta):
+class AbstractQName(AnyAtomicType):
     """
     XPath compliant QName, bound with a prefix and a namespace.
 
@@ -26,7 +27,7 @@ class AbstractQName(metaclass=AtomicTypeMeta):
     def __new__(cls, *args: Any, **kwargs: Any) -> 'AbstractQName':
         if cls.__name__ == 'Notation':
             raise TypeError("can't instantiate xs:NOTATION objects")
-        return cast(AbstractQName, super().__new__(cls))
+        return super().__new__(cls)
 
     def __init__(self, uri: Optional[str], qname: str) -> None:
         if uri is None:
@@ -58,6 +59,10 @@ class AbstractQName(metaclass=AtomicTypeMeta):
     def expanded_name(self) -> str:
         return '{%s}%s' % (self.uri, self.local_name) if self.uri else self.local_name
 
+    @property
+    def braced_uri_name(self) -> str:
+        return 'Q{%s}%s' % (self.uri, self.local_name)
+
     def __repr__(self) -> str:
         return '%s(uri=%r, qname=%r)' % (self.__class__.__name__, self.uri, self.qname)
 
@@ -70,6 +75,8 @@ class AbstractQName(metaclass=AtomicTypeMeta):
     def __eq__(self, other: object) -> bool:
         if isinstance(other, AbstractQName):
             return self.uri == other.uri and self.local_name == other.local_name
+        elif isinstance(other, (str, UntypedAtomic)):
+            return other == self.qname
         raise TypeError("cannot compare {!r} to {!r}".format(type(self), type(other)))
 
 
