@@ -7,7 +7,7 @@
 #
 # @author Davide Brunato <brunato@sissa.it>
 #
-# type: ignore
+# mypy: ignore-errors
 """
 XPath 2.0 implementation - part 4 (XSD constructors)
 """
@@ -18,6 +18,7 @@ from ..datatypes import xsd10_atomic_types, xsd11_atomic_types, GregorianDay, \
     GregorianYearMonth10, GregorianYearMonth, Duration, DayTimeDuration, \
     YearMonthDuration, Date10, Date, DateTime10, DateTime, DateTimeStamp, \
     Time, UntypedAtomic, QName, HexBinary, Base64Binary, BooleanProxy
+from ..xpath_context import XPathSchemaContext
 from ._xpath2_functions import XPath2Parser
 
 
@@ -213,16 +214,22 @@ def cast_time_type(self, value):
 @method('gYearMonth')
 @method('time')
 def evaluate_other_datetime_types(self, context=None):
+    if self.context is not None:
+        context = self.context
+
     arg = self.data_value(self.get_argument(context))
     if arg is None:
         return []
 
     try:
         return self.cast(arg)
-    except TypeError as err:
-        raise self.error('FORG0006', err) from None
-    except OverflowError as err:
-        raise self.error('FODT0001', err) from None
+    except (TypeError, OverflowError) as err:
+        if isinstance(context, XPathSchemaContext):
+            return []
+        elif isinstance(err, TypeError):
+            raise self.error('FORG0006', err) from None
+        else:
+            raise self.error('FODT0001', err) from None
 
 
 ###
@@ -295,6 +302,9 @@ def cast_datetime_stamp_type(self, value):
 
 @method('dateTimeStamp')
 def evaluate_datetime_stamp_type(self, context=None):
+    if self.context is not None:
+        context = self.context
+
     arg = self.data_value(self.get_argument(context))
     if arg is None:
         return []
@@ -315,11 +325,12 @@ def nud_datetime_stamp_type(self):
         self.parser.advance('(')
         self[0:] = self.parser.expression(5),
         if self.parser.next_token.symbol == ',':
-            raise self.wrong_nargs('Too many arguments: expected at most 1 argument')
+            msg = 'Too many arguments: expected at most 1 argument'
+            raise self.error('XPST0017', msg)
         self.parser.advance(')')
         self.value = None
-    except SyntaxError:
-        raise self.error('XPST0017') from None
+    except SyntaxError as err:
+        raise self.error('XPST0017', str(err)) from None
     return self
 
 
@@ -328,7 +339,7 @@ def nud_datetime_stamp_type(self):
 @constructor('base64Binary')
 def cast_base64_binary_type(self, value):
     try:
-        return Base64Binary(value)
+        return Base64Binary(value, ordered=self.parser.version >= '3.1')
     except ValueError as err:
         raise self.error('FORG0001', err) from None
     except TypeError as err:
@@ -338,7 +349,7 @@ def cast_base64_binary_type(self, value):
 @constructor('hexBinary')
 def cast_hex_binary_type(self, value):
     try:
-        return HexBinary(value)
+        return HexBinary(value, ordered=self.parser.version >= '3.1')
     except ValueError as err:
         raise self.error('FORG0001', err) from None
     except TypeError as err:
@@ -348,13 +359,15 @@ def cast_hex_binary_type(self, value):
 @method('base64Binary')
 @method('hexBinary')
 def evaluate_binary_types(self, context=None):
-    arg = self.data_value(self.get_argument(context))
+    arg = self.data_value(self.get_argument(self.context or context))
     if arg is None:
         return []
 
     try:
         return self.cast(arg)
     except ElementPathError as err:
+        if isinstance(context, XPathSchemaContext):
+            return []
         err.token = self
         raise
 
@@ -401,10 +414,12 @@ def cast_boolean_type(self, value):
 def nud_boolean_type_and_function(self):
     self.parser.advance('(')
     if self.parser.next_token.symbol == ')':
-        raise self.wrong_nargs('Too few arguments: expected at least 1 argument')
+        msg = 'Too few arguments: expected at least 1 argument'
+        raise self.error('XPST0017', msg)
     self[0:] = self.parser.expression(5),
     if self.parser.next_token.symbol == ',':
-        raise self.wrong_nargs('Too many arguments: expected at most 1 argument')
+        msg = 'Too many arguments: expected at most 1 argument'
+        raise self.error('XPST0017', msg)
     self.parser.advance(')')
     self.value = None
     return self
@@ -412,6 +427,9 @@ def nud_boolean_type_and_function(self):
 
 @method('boolean')
 def evaluate_boolean_type_and_function(self, context=None):
+    if self.context is not None:
+        context = self.context
+
     if self.label == 'function':
         return self.boolean_value([x for x in self[0].select(context)])
 
@@ -423,6 +441,8 @@ def evaluate_boolean_type_and_function(self, context=None):
     try:
         return self.cast(arg)
     except ElementPathError as err:
+        if isinstance(context, XPathSchemaContext):
+            return []
         err.token = self
         raise
 
@@ -446,8 +466,7 @@ def nud_string_type_and_function(self):
             self[0:] = self.parser.expression(5),
         self.parser.advance(')')
     except ElementPathSyntaxError as err:
-        err.code = self.error_code('XPST0017')
-        raise
+        raise self.error('XPST0017', err)
 
     self.value = None
     return self
@@ -455,6 +474,9 @@ def nud_string_type_and_function(self):
 
 @method('string')
 def evaluate_string_type_and_function(self, context=None):
+    if self.context is not None:
+        context = self.context
+
     if self.label == 'function':
         if not self:
             if context is None:
@@ -531,6 +553,9 @@ def nud_qname_and_datetime(self):
 
 @method('QName')
 def evaluate_qname_type_and_function(self, context=None):
+    if self.context is not None:
+        context = self.context
+
     if self.label == 'constructor function':
         arg = self.data_value(self.get_argument(context))
         return [] if arg is None else self.cast(arg)
@@ -539,14 +564,20 @@ def evaluate_qname_type_and_function(self, context=None):
         qname = self.get_argument(context, index=1)
         try:
             return QName(uri, qname)
-        except TypeError as err:
-            raise self.error('XPTY0004', err)
-        except ValueError as err:
-            raise self.error('FOCA0002', err)
+        except (TypeError, ValueError) as err:
+            if isinstance(context, XPathSchemaContext):
+                return []
+            elif isinstance(err, TypeError):
+                raise self.error('XPTY0004', err)
+            else:
+                raise self.error('FOCA0002', err)
 
 
 @method('dateTime')
 def evaluate_datetime_type_and_function(self, context=None):
+    if self.context is not None:
+        context = self.context
+
     if self.label == 'constructor function':
         arg = self.data_value(self.get_argument(context))
         if arg is None:
@@ -554,10 +585,13 @@ def evaluate_datetime_type_and_function(self, context=None):
 
         try:
             return self.cast(arg)
-        except ValueError as err:
-            raise self.error('FORG0001', err) from None
-        except TypeError as err:
-            raise self.error('FORG0006', err) from None
+        except (ValueError, TypeError) as err:
+            if isinstance(context, XPathSchemaContext):
+                return []
+            elif isinstance(err, ValueError):
+                raise self.error('FORG0001', err) from None
+            else:
+                raise self.error('FORG0006', err) from None
     else:
         dt = self.get_argument(context, cls=Date10)
         tm = self.get_argument(context, 1, cls=Time)
@@ -584,7 +618,7 @@ def cast_untyped_atomic(self, value):
 
 @method('untypedAtomic')
 def evaluate_untyped_atomic(self, context=None):
-    arg = self.data_value(self.get_argument(context))
+    arg = self.data_value(self.get_argument(self.context or context))
     if arg is None:
         return []
     elif isinstance(arg, UntypedAtomic):
