@@ -7,22 +7,34 @@
 #
 # @author Davide Brunato <brunato@sissa.it>
 #
-from typing import Any, Dict, Optional, Iterator, Type
+from typing import TYPE_CHECKING, Any, Dict, Optional, Iterator, Union, Type
 
 from .namespaces import NamespacesType
-from .xpath_context import ContextRootType, XPathContext
-from .xpath1 import XPath1Parser
+from .tree_builders import RootArgType
+from .xpath_context import XPathContext
 from .xpath2 import XPath2Parser
 
-ParserType = Optional[Type[XPath1Parser]]
+if TYPE_CHECKING:
+    from .xpath1 import XPath1Parser
+    from .xpath30 import XPath30Parser
+
+    ParserType = Union[Type[XPath1Parser], Type[XPath2Parser], Type[XPath30Parser]]
+else:
+    ParserType = XPath2Parser
 
 
-def select(root: ContextRootType, path: str, namespaces: NamespacesType = None,
-           parser: ParserType = None, **kwargs: Any) -> Any:
+def select(root: Optional[RootArgType],
+           path: str,
+           namespaces: Optional[NamespacesType] = None,
+           parser: Optional[ParserType] = None,
+           **kwargs: Any) -> Any:
     """
     XPath selector function that apply a *path* expression on *root* Element.
 
-    :param root: an Element or ElementTree instance.
+    :param root: the root of the XML document, usually an ElementTree instance or an \
+    Element. A schema or a schema element can also be provided, or an already built \
+    node tree. You can also provide `None`, in which case no XML root node is set in \
+    the dynamic context, and you have to provide the keyword argument *item*.
     :param path: the XPath expression.
     :param namespaces: a dictionary with mapping from namespace prefixes into URIs.
     :param parser: the parser class to use, that is :class:`XPath2Parser` for default.
@@ -31,6 +43,8 @@ def select(root: ContextRootType, path: str, namespaces: NamespacesType = None,
     on a function or literal.
     """
     context_kwargs = {
+        'uri': kwargs.pop('uri', None),
+        'fragment': kwargs.pop('fragment', False),
         'item': kwargs.pop('item', None),
         'position': kwargs.pop('position', 1),
         'size': kwargs.pop('size', 1),
@@ -41,17 +55,23 @@ def select(root: ContextRootType, path: str, namespaces: NamespacesType = None,
     }
     _parser = (parser or XPath2Parser)(namespaces, **kwargs)
     root_token = _parser.parse(path)
-    context = XPathContext(root, **context_kwargs)
+    context = XPathContext(root, namespaces, **context_kwargs)
     return root_token.get_results(context)
 
 
-def iter_select(root: ContextRootType, path: str, namespaces: NamespacesType = None,
-                parser: ParserType = None, **kwargs: Any) -> Iterator[Any]:
+def iter_select(root: Optional[RootArgType],
+                path: str,
+                namespaces: Optional[NamespacesType] = None,
+                parser: Optional[ParserType] = None,
+                **kwargs: Any) -> Iterator[Any]:
     """
     A function that creates an XPath selector generator for apply a *path* expression
     on *root* Element.
 
-    :param root: an Element or ElementTree instance.
+    :param root: the root of the XML document, usually an ElementTree instance or an \
+    Element. A schema or a schema element can also be provided, or an already built \
+    node tree. You can also provide `None`, in which case no XML root node is set in \
+    the dynamic context, and you have to provide the keyword argument *item*.
     :param path: the XPath expression.
     :param namespaces: a dictionary with mapping from namespace prefixes into URIs.
     :param parser: the parser class to use, that is :class:`XPath2Parser` for default.
@@ -59,6 +79,8 @@ def iter_select(root: ContextRootType, path: str, namespaces: NamespacesType = N
     :return: a generator of the XPath expression results.
     """
     context_kwargs = {
+        'uri': kwargs.pop('uri', None),
+        'fragment': kwargs.pop('fragment', False),
         'item': kwargs.pop('item', None),
         'position': kwargs.pop('position', 1),
         'size': kwargs.pop('size', 1),
@@ -69,7 +91,7 @@ def iter_select(root: ContextRootType, path: str, namespaces: NamespacesType = N
     }
     _parser = (parser or XPath2Parser)(namespaces, **kwargs)
     root_token = _parser.parse(path)
-    context = XPathContext(root, **context_kwargs)
+    context = XPathContext(root, namespaces, **context_kwargs)
     return root_token.select_results(context)
 
 
@@ -90,8 +112,11 @@ class Selector(object):
     :ivar root_token: the root of tokens tree compiled from path.
     :vartype root_token: XPathToken
     """
-    def __init__(self, path: str, namespaces: NamespacesType = None,
-                 parser: ParserType = None, **kwargs: Any) -> None:
+    def __init__(self, path: str,
+                 namespaces: Optional[NamespacesType] = None,
+                 parser: Optional[ParserType] = None,
+                 **kwargs: Any) -> None:
+
         self._variables = kwargs.pop('variables', None)  # For backward compatibility
         self.parser = (parser or XPath2Parser)(namespaces, **kwargs)
         self.path = path
@@ -107,11 +132,12 @@ class Selector(object):
         """A dictionary with mapping from namespace prefixes into URIs."""
         return self.parser.namespaces
 
-    def select(self, root: ContextRootType, **kwargs: Any) -> Any:
+    def select(self, root: Optional[RootArgType], **kwargs: Any) -> Any:
         """
         Applies the instance's XPath expression on *root* Element.
 
-        :param root: an Element or ElementTree instance.
+        :param root: the root of the XML document, usually an ElementTree instance \
+        or an Element.
         :param kwargs: other optional parameters for the XPath dynamic context.
         :return: a list with XPath nodes or a basic type for expressions based on \
         a function or literal.
@@ -122,12 +148,13 @@ class Selector(object):
         context = XPathContext(root, **kwargs)
         return self.root_token.get_results(context)
 
-    def iter_select(self, root: ContextRootType, **kwargs: Any) -> Iterator[Any]:
+    def iter_select(self, root: Optional[RootArgType], **kwargs: Any) -> Iterator[Any]:
         """
         Creates an XPath selector generator for apply the instance's XPath expression
         on *root* Element.
 
-        :param root: an Element or ElementTree instance.
+        :param root: the root of the XML document, usually an ElementTree instance \
+        or an Element.
         :param kwargs: other optional parameters for the XPath dynamic context.
         :return: a generator of the XPath expression results.
         """
